package references

import (
	"context"
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/expr"
	"github.com/stretchr/testify/require"
)

func TestReference_InExpr(t *testing.T) {
	ctx := context.Background()

	tests := []struct {
		exp        string
		env        map[string]interface{}
		wantResult interface{}
		wantErr    bool
	}{
		{"r.id", map[string]interface{}{"r": &Reference{ID: "some_id"}}, "some_id", false},
		{"r.collection_id", map[string]interface{}{"r": &Reference{CollectionID: "some_coll_id"}}, "some_coll_id", false},
		{"r.disabled", map[string]interface{}{"r": &Reference{Disabled: true}}, true, false},
		{"r.String()", map[string]interface{}{"r": &Reference{ID: "id", CollectionID: "collID"}}, "collID.id", false},
		{"r1.Equal(r2)", map[string]interface{}{"r1": &Reference{"id", "collID", false}, "r2": &Reference{"id", "collID", false}}, true, false},
		{"r.IsValid()", map[string]interface{}{"r": &Reference{}}, false, false},
		{"r.not_exists", map[string]interface{}{"r": &Reference{}}, false, true},
	}

	for _, tt := range tests {
		t.Run(tt.exp, func(t *testing.T) {
			result, err := expr.Eval(ctx, tt.exp, tt.env)
			if tt.wantErr {
				require.Error(t, err)
				return
			}

			require.NoError(t, err)
			require.Equal(t, tt.wantResult, result)
		})
	}
}
